function [mt ds rec_out] = calcphasedetect(t, pm, s, rec, cons, fcglobal0, fcglobal1)
%#codegen

ds = zeros(4, 1);
mt = [0; 0];
rec_out = [0; 0];

%% AFO
% states of the oscilator
phi_left = s(1);
phi_right = s(1) + pi;
omega = s(2);
ampl = s(3);
offset = s(4);

% evaluate the adaptive oscilator
teach_signal_left = pm(1) - offset - ampl.*sin(phi_left);
teach_signal_right = pm(2) - offset - ampl.*sin(phi_right);

dphi = omega + cons.eps * teach_signal_left * cos(phi_left) ...
    + cons.eps * teach_signal_right * cos(phi_right);
domega = cons.eps * teach_signal_left * cos(phi_left) ...
    + cons.eps * teach_signal_right * cos(phi_right);
dampl = cons.nu * teach_signal_left * sin(phi_left) + ...
    cons.nu * teach_signal_right * sin(phi_right);
doffset = cons.nu * teach_signal_left + cons.nu * teach_signal_right;

ds(1) = dphi;
ds(2) = domega;
ds(3) = dampl;
ds(4) = doffset;

%% phase offset
lc = length(fcglobal0);
contacts1 = [any(fcglobal1(1:(lc/2))) any(fcglobal1((lc/2+1):end))] * 1.0; % left/right ground contact at timestep N-1
contacts0 = [any(fcglobal0(1:(lc/2))) any(fcglobal0((lc/2+1):end))] * 1.0; % left/right ground contact at timestep N-2

if contacts0(1) < contacts1(1) % first foor makes contact
    rec_out(2) = phi_left;
elseif contacts0(2) < contacts1(2)
    rec_out(2) = phi_right; % second foot makes contact
else
    rec_out(2) = rec(2);
end

rec_out(1) = mod(phi_left - rec_out(2), 2 * pi); % correct the phase signal if ground contacts are detected

end

