function [tm, ds, rec_out] = calcreflex(t, N, pm, q, A, l_CE, s_filt, rec, rec_out, c, r, fcglobal)
%REFLEXCALCTORQUE calculates the outputs of the reflex model

%#codegen

lc = length(fcglobal);
contacts = [any(fcglobal(1:(lc/2))) any(fcglobal((lc/2+1):end))];
refi= [1 2 3; 4 5 6];

theta_f = -r.q_ref_f(3);
dtheta_f = -r.dq_ref_f(3);

if N == 1
    qtheta_pto = [-q(3); -q(3)];
else
    qtheta_pto = rec([29; 30]);
end

% new implementation of the contact force, where the contact force is
% filtered
contactForce = [sum(fcglobal((2:3:(lc/2))))/800, ...
    sum(fcglobal(lc/2 + (2:3:(lc/2))))/800];
ds_Force = [c.filt.A*s_filt(1) + c.filt.B*contactForce(1); ...
    c.filt.A*s_filt(2) + c.filt.B*contactForce(2)];
contactForceFilt = [c.filt.C*s_filt(1) + c.filt.D*contactForce(1); ...
    c.filt.C*s_filt(2) + c.filt.D*contactForce(2)];
    
%% muscle dynamics
% torque vector
nMuscles = 7;
S = zeros(14, 1);

for i = 1:2 % loop left right
    % check left foot contact and reflexes
    iS = (i-1) * nMuscles;
    
    if contacts(i)
        % stance reflexes
        qtheta_pto(i) = -q(3);
        
        % check double support
        not_i = 3-i;
        
        if contacts(not_i) && (pm(refi(i,1)) < pm(refi(not_i,1)))
            DSup = 1;
        else
            DSup = 0;
        end
        
        Fleg_f = contactForceFilt(i);
        Fleg_contra_f = contactForceFilt(not_i);
        
        % calculate muscle stimulation, limit at 1
        S(iS + 1) = c.S_0 ...
            + c.m(1).G * r.refForce_s(iS+1); % soleus
        % tibialis: here the implementatione is slightly different from
        % Geyer. Here the negative feedback is proportional to the average
        % soleus and gastrocnemius torque instead of only the soleus
        % torque.
        S(iS + 2) = c.S_0 ...
            + max(0, c.m(2).G * (r.l_CE_s(iS+2)-c.m(2).l_off)) ...
            - max(0, c.m(2).G2 * (r.refForce_s(iS+1) + r.refForce_s(iS+3))/2); % tibialis anterior (NEW)
%         S(iS + 2) = c.S_0 ...
%             + max(0, c.m(2).G * (r.l_CE_s(iS+2)-c.m(2).l_off)) ...
%             - max(0, c.m(2).G2 * r.refForce_s(iS+1)); % tibialis anterior (ORIGINAL)
        S(iS + 3) = c.S_0 ...
            + c.m(3).G * r.refForce_s(iS+3); % gastrocnemius
        S(iS + 4) = c.m(4).S_0 ...
            + c.m(4).G * r.refForce_m(iS+4) ...
            - c.k_phi ...
            * (r.qrel_ref_m(refi(i,2)) - c.phi_knee_off)...
            * (r.qrel_ref_m(refi(i,2)) > c.phi_knee_off)...
            * (r.dqrel_ref_m(refi(i,2)) > 0) ...
            - c.k_bw * abs(Fleg_contra_f) * DSup; % vastus
        S(iS + 5) = c.S_0_bal +  (...
            c.k_p * (theta_f - c.theta_ref) ...
            + c.k_d * dtheta_f) ...
            * c.k_bw * abs(Fleg_f); % hamstring
        S(iS + 6) =  c.S_0_bal + ...
            max(0, ...
            0.68 * c.k_p * (theta_f - c.theta_ref) ...
            + c.k_d * dtheta_f) ...
            * c.k_bw * abs(Fleg_f) ...
            - c.dS * DSup; % gluteus
        S(iS + 7) = (c.S_0_bal ...
            - min(0, c.k_p * (theta_f - c.theta_ref) ...
            + c.k_d * dtheta_f)) ...
            * c.k_bw * abs(Fleg_f) ...
            + c.dS * DSup; % hip flexor
    else
        % swing reflexes
        S(iS + 1) = c.S_0; % soleus
        S(iS + 2) = c.S_0 ...
            + c.m(2).G * (r.l_CE_s(iS+2)-c.m(2).l_off); % tibialis anterior
        S(iS + 3) = c.S_0; % gastrocnemius
        S(iS + 4) = c.S_0; % vastus
        S(iS + 5) = c.S_0 ...
            + c.m(5).G * r.refForce_f(iS+5); % hamstring
        S(iS + 6) = c.S_0 ...
            + c.m(6).G * r.refForce_f(iS+6); % gluteus
        S(iS + 7) = c.S_0 ...
            + max(0, c.m(7).G * (r.l_CE_f(iS+7)-c.m(7).l_off)) ...
            - max(0, c.m(7).G2 * (r.l_CE_f(iS+5)-c.m(5).l_off)) ...
            + c.k_lean * (qtheta_pto(i) - c.theta_ref); % hip flexor
    end % stance swing reflexes
end % loop left right

musclenoise = rec_out(31:44) * c.maxnoise;
S = S + S .* musclenoise;

S = min(1, max(0.01, S));
if t <= c.dt_f
    S(:) = 0.01;
elseif t <= c.dt_m
    S([1 2 3 4 8 9 10 11]) = 0.01;
elseif t <= c.dt_s
    S([1 2 3 8 9 10]) = 0.01;
end

[v_CE, dA, F_m_out, F_CE, tm] = loopMuscles(c.m, pm(refi), l_CE, A, S);
tm = -tm;

% extra state derivative output
ds = [dA; v_CE; ds_Force];

rec_out(1:30) = [F_m_out; F_CE; qtheta_pto];
rec_out(45:50) = tm;

end

