classdef ffgaitpattern < controller
    %FFGAITPATTERN This is a controller class for puting a feed forward
    %torque on the human joints
    
    properties
        gaitphase = 0;
    end
    
    methods
        function c = ffgaitpattern()
            c.name = 'ff_gaitpattern';
            c.rec = 6;
            
            c.cons.on = 0; % on/off switch for the control
            c.cons.tOn = 0; % on time for the controlles
            c.cons.tSwitch = 0.2; % transition time between on and of
            
            W = load('winter');
            c.cons.w = W.W; % assign the winter torques the the controller constants
            c.cons.weight = 80; % weight of the subject
            c.cons.winterjoints = [1 2 3]; % joints to be actuated [hip knee ankle]
        end
        
        function writebody(c, file_id, mdl)
            % write the body of the controller
            fprintf(file_id, ['s = s_in(' mat2str(c.sid) '); \n']);
            % get the gait phase
            fprintf(file_id, ['phase = rec_in(' mat2str(c.gaitphase.recid(1)) ', max(1, N-1)); \n']);
            % calculate and apply the torques
            fprintf(file_id, '[mt rec_out] = calcffgaitpattern(phase, t, cons); \n');
        end
        
        function initsim(c, ~)
        end
        
        function update(c, ~)
            nkern = c.cons.filter.nkern;
            
            dc = pi / nkern;
            
            c.cons.filter.c = linspace(dc, 2 * pi - dc, nkern)';
            c.cons.filter.hpre = c.cons.filter.h * nkern;

            % Left right symmetric
            % Shared phase and frequency state for all oscillators
            %   - p: phase of the oscillator
            %   - w: frequency of the oscillator

            % States foreach each pair of L/R are:
            %   - w[1 x nkern]: weights of the gaussian
            c.s = 4 + nkern * (length(c.motors) / 2);
            
            % Record desired position and velocity for each motor as well as
            % torques
            c.rec = length(c.motors) * 3;
            
            % All initial states to 0
            c.s0 = zeros(c.s, 1);
            
            % Set initial frequency to something like normal walking
            c.s0(2) = 1.2 * (2 * pi);
            
            if c.cons.limits.clip
                c.cons.k = min(c.cons.k, c.cons.limits.stiffness);
                c.cons.d = min(c.cons.d, c.cons.limits.damping);
            end
        end
    end
    
end

