classdef reflex < controller
    %REFLEX Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        contacts
    end
    
    methods
        function c = reflex()
            c.name = 'reflex';
            c.s = 30; % 14 stimulations, 14 muscle lengths, 2 low pass force filters
            c.rec = 50;
            c.cons = initreflex;
            c.cons.dt_s  = 0.02; % slow reflex time [s]
            c.cons.dt_m  = 0.01; % medium reflex time [s]
            c.cons.dt_f  = 0.005; % fast reflex time [s]
            c.cons.S_0   = 0.01; % baseline stimulation
            c.cons.S_0_bal = 0.01; % baseline stimulation balistic reflex
            c.cons.k_phi = 2; % knee angle gain
            c.cons.k_bw = 1.2; % double support gain
            c.cons.k_p = 1.91; % lean angle trunk p-gain
            c.cons.k_d = 0.25; % lean angle trunk d-gain
            c.cons.k_lean = 1.15; % lean angle gain
            c.cons.phi_knee_off = -0.1; % knee offset [rad]
            c.cons.theta_ref = 0.1047; % refernece lean angle trunk [rad]
            c.cons.dS = 0.25; % stimulation offset
            c.cons.maxnoise = 0;
            c.cons.noiseX = [];
            c.cons.noiseY = [];
            [c.cons.filt.A, ...
                c.cons.filt.B, ...
                c.cons.filt.C, ...
                c.cons.filt.D] = butter(1,5*2*pi,'low','s');
        end
        
        function writebody(c, file_id, mdl)
            %% reference states
            fprintf(file_id, 'rec_s = interpfast(t - cons.dt_s, dt, t_all, rec_in); \n');
            fprintf(file_id, 'rec_m = interpfast(t - cons.dt_m, dt, t_all, rec_in); \n');
            fprintf(file_id, 'rec_f = interpfast(t - cons.dt_f, dt, t_all, rec_in); \n');

            fprintf(file_id, 'x_s = interpfast(t - cons.dt_s, dt, t_all, x_all); \n');
            fprintf(file_id, 'x_m = interpfast(t - cons.dt_m, dt, t_all, x_all); \n');
            fprintf(file_id, 'x_f = interpfast(t - cons.dt_f, dt, t_all, x_all); \n');
            
            l_CEid = (mdl.nq + sum(~mdl.q_dep)) + c.sid(15:28);
            
            fprintf(file_id, ['r.q_ref_s = x_s(' mat2str(1:mdl.nq) '); \n']);
            fprintf(file_id, ['r.dq_ref_s = x_s(' mat2str((mdl.nq + 1):(mdl.nq * 2)) '); \n']);
            fprintf(file_id, ['r.l_CE_s = x_s(' mat2str(l_CEid) '); \n']);
            
            fprintf(file_id, ['r.q_ref_m = x_s(' mat2str(1:mdl.nq) '); \n']);
            fprintf(file_id, ['r.dq_ref_m = x_s(' mat2str((mdl.nq + 1):(mdl.nq * 2)) '); \n']);
            fprintf(file_id, ['r.l_CE_m = x_s(' mat2str(l_CEid) '); \n']);
            
            fprintf(file_id, ['r.q_ref_f = x_s(' mat2str(1:mdl.nq) '); \n']);
            fprintf(file_id, ['r.dq_ref_f = x_s(' mat2str((mdl.nq + 1):(mdl.nq * 2)) '); \n']);
            fprintf(file_id, ['r.l_CE_f = x_s(' mat2str(l_CEid) '); \n']);
            
            fprintf(file_id, ['r.refForce_s = rec_s(' mat2str(c.recid) '); \n']);
            fprintf(file_id, ['r.refForce_m = rec_m(' mat2str(c.recid) '); \n']);
            fprintf(file_id, ['r.refForce_f = rec_f(' mat2str(c.recid) '); \n']);
            
            % loop all the motors
            fprintf(file_id, ['r.qrel_ref_m = zeros(' num2str(length(c.motors)) ',1); \n']);
            fprintf(file_id, ['r.dqrel_ref_m = zeros(' num2str(length(c.motors)) ',1); \n']);
            for iM = 1:length(c.motors)
                if ~isempty(c.motors{iM}.parent.qid)
                    fprintf(file_id, ['r.qrel_ref_m(' num2str(iM) ') = r.q_ref_m(' num2str(c.motors{iM}.qid) ') - r.q_ref_m(' num2str(c.motors{iM}.parent.qid(end)) '); \n']);
                    fprintf(file_id, ['r.dqrel_ref_m(' num2str(iM) ') = r.dq_ref_m(' num2str(c.motors{iM}.qid) ') - r.dq_ref_m(' num2str(c.motors{iM}.parent.qid(end)) '); \n']);
                else
                    fprintf(file_id, ['r.qrel_ref_m(' num2str(iM) ') = r.q_ref_m(' num2str(c.motors{iM}.qid) '); \n']);
                    fprintf(file_id, ['r.dqrel_ref_m(' num2str(iM) ') = r.dq_ref_m(' num2str(c.motors{iM}.qid) '); \n']);
                end
            end
            fprintf(file_id, ['A = s_in(' mat2str(c.sid(1:14)) '); \n']);
            fprintf(file_id, ['l_CE = s_in(' mat2str(c.sid(15:28)) '); \n']);
            fprintf(file_id, ['s_filt = s_in(' mat2str(c.sid(29:30)) '); \n']);
            fprintf(file_id, ['fcglobal = rec_in(' mat2str(c.contacts.recid) ', max(1, N-1)); \n']);
            fprintf(file_id, ['rec = rec_in(' mat2str(c.recid) ', max(1, N-1)); \n']);
            fprintf(file_id, ['rec_out = rec_in(' mat2str(c.recid) ', N); \n']);
            fprintf(file_id, '[mt ds rec_out] = calcreflex(t, N, pm, q, A, l_CE, s_filt, rec, rec_out, cons, r, fcglobal); \n');
        end
        
        function c = initsim(c, mdl)
            c.s0 = [0.01; 0.01; 0.01; 0.01; 0.01; 0.01; 0.01; ... % activation t_0
                0.01; 0.01; 0.01; 0.01; 0.01; 0.01; 0.01; ... % activation t_0
                0.04; 0.06; 0.05; 0.08; 0.1; 0.11; 0.11;... % CE length t_0
                0.04; 0.06; 0.05; 0.08; 0.1; 0.11; 0.11;... % CE length t_0
                0; 0]; % initial state force filters
            
            q0rel = zeros(6,1);
            refi = [1 2 3; 4 5 6];
            for iM = 1:length(c.motors)
                if ~isempty(c.motors{iM}.parent.qid)
                    q0rel(iM) = mdl.x0(c.motors{iM}.qid) - mdl.x0(c.motors{iM}.parent.qid(end));
                else
                    q0rel(iM) = mdl.x0(c.motors{iM}.qid);
                end
            end
            
            for ileg = 1:2
                for imuscle = 1:7
                    phi = q0rel(refi(ileg,:));
                    mus = c.cons.m(imuscle);
                    dl_MTU = mus.dir * sum([...
                        mus.rho*mus.r0(1) * (phi(1)-mus.phi_ref(1)); ...
                        mus.rho*mus.r0(2) * (sin(phi(2) - mus.phi_max(2)) - sin(mus.phi_ref(2) - mus.phi_max(2))); ...
                        mus.rho*mus.r0(3) * (sin(phi(3) - mus.phi_max(3)) - sin(mus.phi_ref(3) - mus.phi_max(3)))]); % deflection of the muscle
                    c.s0(14 + (ileg - 1) * 7 + imuscle) = dl_MTU + mus.l_opt;
                end
            end
            
            range = mdl.t(1):mdl.dt:mdl.t(2);
            c.rec0 = zeros(c.rec, length(range));
            
            if ~isempty(c.cons.noiseX)
                if c.cons.noiseX < mdl.t(2)
                    disp('WARNING: Not noise vector shorter than time vector')
                end
                for iNoise = 1:14
                    c.rec0(iNoise + 30,:) = pchip(c.cons.noiseX(iNoise, :),c.cons.noiseY(iNoise, :),range);
                end
            end
            
            % tijdelijke toevoegin voor betere loop resultaten
            c.s0(1) = c.s0(1) - 0.005;
            c.s0(3) = c.s0(3) - 0.015;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        end % initsim
    end % methods
end
