classdef controller < handle
    %CONTROLLER Summary of this class goes here
    %   Detailed explanation goes here
    
    properties (SetObservable)
        motors % joints to which the controller applies
        points % points to which external forces applies
        cid % controller id
        cons % constants that are used by the controller
        s = 0 % controler states that are integrated
        s0 = [] % initial state
        sid % states id
        rec = 0 % output values that are recorded in the controller
        rec0 = [] % first output of the recorded state
        recid % recorded output id
        name % name of the control
    end
    
    methods
        function writecontroller(c, ctype, mdl)
            % make the T matrix
            file_id = fopen(fullfile('generated', [ctype.name '_control.m']), 'w'); % open file
            fprintf(file_id, ['function [F ds rec_out] = ' c.name '_control(t, N, dt, q, dq, t_all, x_all, cons, s_in, rec_in, Tj, Tm, Tp, dTj, dTm, dTp, M)\n']); % write file header
            fprintf(file_id, '%%#codegen \n');
            fprintf(file_id, ['F = zeros(' num2str(mdl.nq) ', 1); \n']);
            
            % loop all the motors
            fprintf(file_id, ['pm = zeros(' num2str(length(c.motors)) ',1); \n']);
            fprintf(file_id, ['vm = zeros(' num2str(length(c.motors)) ',1); \n']);
            for iM = 1:length(c.motors)
                if ~isempty(c.motors{iM}.parent.qid)
                    fprintf(file_id, ['pm(' num2str(iM) ') = q(' num2str(c.motors{iM}.qid) ') - q(' num2str(c.motors{iM}.parent.qid(end)) '); \n']);
                    fprintf(file_id, ['vm(' num2str(iM) ') = dq(' num2str(c.motors{iM}.qid) ') - dq(' num2str(c.motors{iM}.parent.qid(end)) '); \n']);
                else
                    fprintf(file_id, ['pm(' num2str(iM) ') = q(' num2str(c.motors{iM}.qid) '); \n']);
                    fprintf(file_id, ['vm(' num2str(iM) ') = dq(' num2str(c.motors{iM}.qid) '); \n']);
                end
            end
            
            ctype.writebody(file_id, mdl);
            
            % loop all the motors
            for iM = 1:length(c.motors)
                fprintf(file_id, ['F(' num2str(c.motors{iM}.qid) ') = F(' num2str(c.motors{iM}.qid) ') + mt(' num2str(iM) '); \n']);
                if ~isempty(c.motors{iM}.parent.qid)
                    fprintf(file_id, ['F(' num2str(c.motors{iM}.parent.qid(end)) ') = F(' num2str(c.motors{iM}.parent.qid(end)) ') - mt(' num2str(iM) '); \n']);
                end
            end
            
            if c.s == 0;
                fprintf(file_id, 'ds = []; \n');
            end
            if c.rec == 0;
                fprintf(file_id, 'rec_out = []; \n');
            end
            
            fclose(file_id);
        end
    end
    
end

