function makeTmatrices(mdl)
%MAKEMATRICES Summary of this function goes here
%   Detailed explanation goes here

file_id = fopen(fullfile('generated',[mdl.prefix 'Tmatrices.m']), 'w'); % open file

%% write the header
fprintf(file_id, 'function [Tj Tm Tp dTj dTm dTp ddTm M] = Tmatrices(q, dq, p_in)\n'); % write file header
fprintf(file_id, '%%#codegen \n');
%% process the inputs
% process the q and dq input
nq = 0;
for ijoint = mdl.joints
    for iq = 1:length(mdl.parts{ijoint}.q);
        nq = nq + 1;
        fprintf(file_id, [mdl.parts{ijoint}.q{iq} ' = q(' num2str(nq) '); \n']);
        fprintf(file_id, [mdl.parts{ijoint}.dq{iq} ' = dq(' num2str(nq) '); \n']);
    end
end

% process the initial positions of the joints (t_in, m_in, p_in)
jmp = [mdl.joints mdl.masses mdl.points];
for ipart = 1:length(jmp)
    ijmp = jmp(ipart);
    fprintf(file_id, ['l_' mdl.parts{ijmp}.name ' = p_in(' num2str(ipart) ', 1); \n']);
    fprintf(file_id, ['theta_' mdl.parts{ijmp}.name ' = p_in(' num2str(ipart) ', 2); \n']);
end

%% make the empty matrices
fprintf(file_id, ['Tj = zeros(' num2str(length(mdl.joints) * 3) ',1);\n']);
fprintf(file_id, ['Tm = zeros(' num2str(length(mdl.masses) * 3) ',1);\n']);
fprintf(file_id, ['Tp = zeros(' num2str(length(mdl.points) * 3) ',1);\n']);
fprintf(file_id, ['dTj = zeros(' num2str(length(mdl.joints) * 3) ', ' num2str(nq) ');\n']);
fprintf(file_id, ['dTm = zeros(' num2str(length(mdl.masses) * 3) ', ' num2str(nq) ');\n']);
fprintf(file_id, ['dTp = zeros(' num2str(length(mdl.points) * 3) ', ' num2str(nq) ');\n']);
fprintf(file_id, ['ddTm = zeros(' num2str(length(mdl.masses) * 3) ', ' num2str(nq) ');\n']);

%% the T matricess
for ipart = [mdl.joints mdl.masses mdl.points]
    jpart = mdl.parts{ipart};
    fprintf(file_id, [jpart.name ' = [' jpart.getsympos{1} '; ' jpart.getsympos{2} '; ' jpart.getsympos{3} ']; \n']);
end

for ijoint = 1:length(mdl.joints)
    jpart = mdl.parts{mdl.joints(ijoint)};
    segsum = '';
    while ~strcmp(jpart.type, 'WORLD')
        if ~isempty(segsum)
            segsum = [segsum ' + '];
        end
        segsum = [segsum jpart.name];
        jpart = jpart.parent;
    end
    fprintf(file_id, ['Tj(' num2str(ijoint * 3 - 2) ':' num2str(ijoint * 3) ') = ' segsum ';\n']);
end

for imass = 1:length(mdl.masses)
    jpart = mdl.parts{mdl.masses(imass)};
    segsum = '';
    while ~strcmp(jpart.type, 'WORLD')
        if ~isempty(segsum)
            segsum = [segsum ' + '];
        end
        segsum = [segsum jpart.name];
        jpart = jpart.parent;
    end
    fprintf(file_id, ['Tm(' num2str(imass * 3 - 2) ':' num2str(imass * 3) ') = ' segsum ';\n']);
end

for ipoint = 1:length(mdl.points)
    jpart = mdl.parts{mdl.points(ipoint)};
    segsum = '';
    while ~strcmp(jpart.type, 'WORLD')
        if ~isempty(segsum)
            segsum = [segsum ' + '];
        end
        segsum = [segsum jpart.name];
        jpart = jpart.parent;
    end
    fprintf(file_id, ['Tp(' num2str(ipoint * 3 - 2) ':' num2str(ipoint * 3) ') = ' segsum ';\n']);
end

%% the dT matrices
for ipart = [mdl.joints mdl.masses mdl.points]
    jpart = mdl.parts{ipart};
    for iq = 1:length(jpart.parent.q)
        fullid = [jpart.name '_' jpart.parent.q{iq}];
        fprintf(file_id, [fullid ' = [' jpart.getsymvel{1, iq} '; ' jpart.getsymvel{2, iq} '; ' jpart.getsymvel{3, iq} ']; \n']);
    end
end

for ijoint = 1:length(mdl.joints)
    jpart = mdl.parts{mdl.joints(ijoint)};
    while ~strcmp(jpart.type, 'WORLD')
        for iq = 1:length(jpart.parent.q)
            fullid = [jpart.name '_' jpart.parent.q{iq}];
            fprintf(file_id, ['dTj(' num2str(ijoint * 3 - 2) ':' num2str(ijoint * 3) ', ' num2str(jpart.parent.qid(iq)) ') = ' fullid ';\n']);
        end
        jpart = jpart.parent;
    end
end

for imass = 1:length(mdl.masses)
    jpart = mdl.parts{mdl.masses(imass)};
    while ~strcmp(jpart.type, 'WORLD')
        for iq = 1:length(jpart.parent.q)
            fullid = [jpart.name '_' jpart.parent.q{iq}];
            fprintf(file_id, ['dTm(' num2str(imass * 3 - 2) ':' num2str(imass * 3) ', ' num2str(jpart.parent.qid(iq)) ') = ' fullid ';\n']);
        end
        jpart = jpart.parent;
    end
end

for ipoint = 1:length(mdl.points)
    jpart = mdl.parts{mdl.points(ipoint)};
    while ~strcmp(jpart.type, 'WORLD')
        for iq = 1:length(jpart.parent.q)
            fullid = [jpart.name '_' jpart.parent.q{iq}];
            fprintf(file_id, ['dTp(' num2str(ipoint * 3 - 2) ':' num2str(ipoint * 3) ', ' num2str(jpart.parent.qid(iq)) ') = ' fullid ';\n']);
        end
        jpart = jpart.parent;
    end
end

%% the ddT marix
for ipart = [mdl.joints mdl.masses]
    jpart = mdl.parts{ipart};
    for iq = 1:length(jpart.parent.q)
        fullid = [jpart.name '_' jpart.parent.q{iq}];
        fprintf(file_id, [fullid ' = [' jpart.getsymacc{1, iq} '; ' jpart.getsymacc{2, iq} '; ' jpart.getsymacc{3, iq} ']; \n']);
    end
end

for imass = 1:length(mdl.masses)
    jpart = mdl.parts{mdl.masses(imass)};
    while ~strcmp(jpart.type, 'WORLD')
        for iq = 1:length(jpart.parent.q)
            fullid = [jpart.name '_' jpart.parent.q{iq}];
            fprintf(file_id, ['ddTm(' num2str(imass * 3 - 2) ':' num2str(imass * 3) ', ' num2str(jpart.parent.qid(iq)) ') = ' fullid ';\n']);
        end
        jpart = jpart.parent;
    end
end

fclose(file_id);
end

