function make_eom(mdl)
%WRITE_EOM this function creates a new file that calculates the equations
%of motion

file_id = fopen(fullfile('generated',[mdl.prefix 'equations_of_motion.m']), 'w'); % open file
fprintf(file_id, 'function [dx rec_out events] = equations_of_motion(t, x, t_all, x_all, rec, N, dt, simcons) \n'); % file header
fprintf(file_id, '%%#codegen \n'); % check code generation errors

%% assign the states
if ~isempty(mdl.constraints)  
    fprintf(file_id, ['q_dep = ' mat2str(find(mdl.q_dep)) '; \n']);
    fprintf(file_id, ['q_ind = ' mat2str(find(~mdl.q_dep)) '; \n']);
end
fprintf(file_id, ['q = x(1:' num2str(mdl.nq) '); \n']);
fprintf(file_id, ['dq = zeros(' num2str(mdl.nq) ',1); \n']);
if ~isempty(mdl.constraints)
    fprintf(file_id, ['dq(q_ind) = x(' num2str(mdl.nq + 1) ':' num2str(mdl.nq + sum(~mdl.q_dep)) '); \n']);
else
    fprintf(file_id, ['dq = x(' num2str(mdl.nq + 1) ':' num2str(mdl.nq + sum(~mdl.q_dep)) '); \n']);
end
if mdl.ns ~= 0
    fprintf(file_id, ['s = x(' num2str(mdl.nq + sum(~mdl.q_dep) + 1) ':' ...
        num2str(mdl.nq + sum(~mdl.q_dep) + mdl.ns) '); \n']);
else
    fprintf(file_id, 's = []; \n'); 
end
fprintf(file_id, '\n');

if ~isempty(mdl.constraints)
    fprintf(file_id, '[dDc ddDc] = Cmatrices(q, dq, simcons.p_in, simcons.c_in); \n\n');

    fprintf(file_id, ['dCm = ' mat2str(zeros(mdl.nq, sum(~mdl.q_dep))) '; \n']);
    fprintf(file_id, ['ddCm = ' mat2str(zeros(mdl.nq, mdl.nq)) '; \n']);
    fprintf(file_id, ['dCm(q_ind, :) = ' mat2str(eye(sum(~mdl.q_dep))) '; \n']);
    fprintf(file_id, 'dCm(q_dep, :) = dDc(:,q_dep)\\-dDc(:,q_ind); \n');
    fprintf(file_id, 'dq = dCm * dq(q_ind); \n');
    
    fprintf(file_id, '[dDc ddDc] = Cmatrices(q, dq, simcons.p_in, simcons.c_in); \n\n');
    fprintf(file_id, 'ddCm(q_dep, :) = dDc(:,q_dep)\\-ddDc; \n');
end

fprintf(file_id, '[Tj Tm Tp dTj dTm dTp ddTm] = Tmatrices(q, dq, simcons.p_in); \n\n');

fprintf(file_id, ['F = zeros(' num2str(mdl.nq) ', 1); \n']);
if mdl.ns ~= 0
    fprintf(file_id, ['ds = zeros(' num2str(mdl.ns) ', 1); \n']);
else
    fprintf(file_id, 'ds = []; \n');
end
if mdl.nrec ~= 0
    fprintf(file_id, ['rec_out = zeros(' num2str(mdl.nrec) ', 1); \n\n']);
else
    fprintf(file_id, 'rec_out = 0; \n');
end

fprintf(file_id, ['events = zeros(' num2str(max(1, length(mdl.simevents))) ', 1); \n']);



%% loop the controllers
for i = 1:length(mdl.controllers)
    if mdl.controllers{i}.s ~= 0
        dsstr = ['ds(' mat2str(mdl.controllers{i}.sid) ')'];
    else
        dsstr = '~';
    end
    if mdl.controllers{i}.rec ~= 0
        rec_outstr = ['rec_out(' mat2str(mdl.controllers{i}.recid) ')'];
    else
        rec_outstr = '~';
    end
    % fprintf(file_id, ['[Fc, ' dsstr ', ' rec_outstr '] = ' mdl.controllers{i}.name '_control(t, N, dt, q, dq, t_all, q_all, dq_all, s_all, simcons.control.c' num2str(i) ', s, rec, Tj, Tm, Tp, dTj, dTm, dTp, M); \n']);
    fprintf(file_id, ['[Fc, ' dsstr ', ' rec_outstr '] = ' mdl.controllers{i}.name '_control(t, N, dt, q, dq, t_all, x_all, simcons.control.c' num2str(i) ', s, rec, Tj, Tm, Tp, dTj, dTm, dTp, simcons.M); \n']);
    fprintf(file_id, 'F = F + Fc; \n');
end

%% equations of motion
fprintf(file_id, '\n');
if isempty(mdl.constraints)
    fprintf(file_id, ['ddq = (dTm'' * simcons.M * dTm) \\ (F - dTm'' * simcons.M * (ddTm * dq)); \n']);
else
    fprintf(file_id, ['ddq = (dCm'' * dTm'' * simcons.M * dTm * dCm) \\ (dCm'' * (F - dTm'' * (simcons.M * (ddTm * dq + dTm * ddCm * dq)))); \n']);
end
fprintf(file_id, 'dx = [dq; ddq; ds]; \n\n');

%% events
for i = 1:length(mdl.simevents)
    fprintf(file_id, ['events(' num2str(i) ') = ' mdl.simevents{i}.name '_event(t, N, dt, q, dq, t_all, x_all, s, rec, Tj, Tm, Tp, dTj, dTm, dTp, simcons.M); \n']);
end

fclose(file_id);
end

