function mdl = preprocessm(mdl)
%PREPROCESS preprocesses the model obeject so it is ready for simulation

%% sort the parts
mdl.worlds = [];
mdl.joints = [];
mdl.masses = [];
mdl.points = [];
mdl.constraints = [];
for ipart = 1:length(mdl.parts)
    switch mdl.parts{ipart}.type
        case 'WORLD'
            mdl.worlds = [mdl.worlds ipart];
        case 'JOINT'
            mdl.joints = [mdl.joints ipart];
        case 'MASS'
            mdl.masses = [mdl.masses ipart];
        case 'POINT'
            mdl.points = [mdl.points ipart];
        case 'CONSTRAINT'
            mdl.constraints = [mdl.constraints ipart];
    end
end

%% assign an index to all the joints and count coordinates
mdl.nq = 0;
for i = 1:length(mdl.joints)
    ijoint = mdl.joints(i);
    for iq = 1:length(mdl.parts{ijoint}.q);
        mdl.nq = mdl.nq + 1;
        mdl.parts{ijoint}.qid(iq) = mdl.nq;
        mdl.parts{ijoint}.Tid = i;
    end
end

%% seperate the dependent and the independent coordinats
if isempty(mdl.q_dep)
    mdl.q_dep = zeros(mdl.nq, 1);
    for icons = mdl.constraints
        mdl.q_dep(mdl.parts{icons}.child.qid(end)) = 1;
        mdl.q_dep(mdl.parts{icons}.parent.qid(end)) = 1;
    end
end

%% preprocess the controllers
mdl.nrec = 0;
mdl.ns = 0;

for icontroller = 1:length(mdl.controllers)   
    mdl.controllers{icontroller}.sid = mdl.ns + [1 : mdl.controllers{icontroller}.s];
    mdl.ns = mdl.ns + mdl.controllers{icontroller}.s;
    
    mdl.controllers{icontroller}.cid = icontroller;
    
    mdl.controllers{icontroller}.recid = mdl.nrec + [1 : mdl.controllers{icontroller}.rec];
    mdl.nrec = mdl.nrec + mdl.controllers{icontroller}.rec;
    
    mdl.controllers{icontroller}.writecontroller(mdl.controllers{icontroller}, mdl);
end

for ievent = 1:length(mdl.simevents)   
    mdl.simevents{ievent}.writecontroller(mdl.simevents{ievent}, mdl);
end

makeTmatrices(mdl); % make a file for all the matrices
clear Tmatrices
if ~isempty(mdl.constraints)
    makeCmatrices(mdl); % make a file for all the matrice
    clear Cmatrices
end

make_eom(mdl); % make a file for the equations of motion
clear equations_of_motion

end

