function mdl = solveconstraints(mdl)
%SOLVECONSTRAINTS solves the initial positions/velocities
%   Note: the functions requires the contraints to be in the order that
%   they can be kinematically solved

for icons = 1:length(mdl.constraints)
    %% positions
    Tj = Tmatrices(mdl.x0 * 0, mdl.dx0 * 0, mdl.simcons.p_in);

    pos1 = Tj(mdl.parts{mdl.constraints(icons)}.parent.Tid * 3 - [2 1]);
    pos2 = Tj(mdl.parts{mdl.constraints(icons)}.child.Tid * 3 - [2 1]);
    
    q_init = [mdl.simcons.c_in(icons, 2) mdl.simcons.c_in(icons, 4); ...
        mdl.simcons.c_in(icons, 2) mdl.simcons.c_in(icons, 4)]; % angles as set in the model definition
    q_kin = closechain(pos1, ...
        pos2, ...
        mdl.simcons.c_in(icons, 1), ...
        mdl.simcons.c_in(icons, 3)); % angles as result of the kinematic solver
    q_err = q_kin - q_init;
    q_err = abs((q_err/pi)-round(q_err/pi));
    [~, s] = min(sum(q_err, 2));
    
    Tj = Tmatrices(mdl.x0, mdl.dx0, mdl.simcons.p_in);
    pos1 = Tj(mdl.parts{mdl.constraints(icons)}.parent.Tid * 3 - [2 1]);
    pos2 = Tj(mdl.parts{mdl.constraints(icons)}.child.Tid * 3 - [2 1]);
    q_kin = closechain(pos1, ...
        pos2, ...
        mdl.simcons.c_in(icons, 1), ...
        mdl.simcons.c_in(icons, 3)); % angles as result of the kinematic solver
    mdl.x0(mdl.parts{mdl.constraints(icons)}.parent.qid) = q_kin(s, 1) - mdl.simcons.c_in(icons, 2);
    mdl.x0(mdl.parts{mdl.constraints(icons)}.child.qid) = q_kin(s, 2) - mdl.simcons.c_in(icons, 4);
end

%% additional funtions
    function q = closechain(A1, A2, l1, l2)
        % calculate angles
        A12 = A2 - A1;
        
        l = (A12(1) ^ 2 + A12(2) ^ 2) ^ 0.5;
        theta = atan2(-A12(1), A12(2));
               
        a112 = cosinerule(l2,l1,l);
        a212 = cosinerule(l1,l2,l);
        
        q = zeros(2,1);
        
        q(1,1) = theta + a112;
        q(1,2) = theta - pi - a212;
        q(2,1) = theta - a112;
        q(2,2) = theta - pi + a212;
    end

    function A = cosinerule(a,b,c)
        % calculates the cosine rule
        A = acos((b^2 + c^2 - a^2)/(2*b*c));
    end

end
