function animator(fig,varargin)
if ischar(varargin{1}) % call to subfunction
    fun = str2func(varargin{1});
    varargin = varargin(2:end);
    fun(fig,varargin{:});
else % create call
    animator_openingFcn(fig,varargin{:});
end

%% animator_openingFcn %%
function animator_openingFcn(fig,plot_fun,tspan,varargin)
% handle options
handles = guidata(fig);
handles.options = animator_options(varargin{:});

% add data to handles
handles.plot_fun = plot_fun;
handles.tspan = tspan;

if isempty(handles.options.UI)
    % create fig panel and move figure objects into panel
    children = get(fig,'Children');
    set(fig,'Units','pixels');
    fig_pos = get(fig,'Position');
    fig_color = get(fig,'Color');
    handles.figpanel = uipanel('Parent',fig,'Units','pixels','Position',fig_pos,'BorderType','none','BackgroundColor',fig_color);
    set(children,'Parent',handles.figpanel);
    
    % create ui panel
    handles.uipanel_height = 40; % in pixels
    %set(fig,'Position',fig_pos+[0,-handles.uipanel_height,0,handles.uipanel_height]); % expand fig
    set(handles.figpanel,'Position',[0,handles.uipanel_height,fig_pos(3),fig_pos(4)-handles.uipanel_height],'BorderWidth',1); % move fig panel
    handles.panel = uipanel('Parent',fig,'Units','pixels','Position',[0,0,fig_pos(3),handles.uipanel_height],'BorderType','none','BorderWidth',1);
    
    % create ui elements
    handles.startbutton_width = 50; % in pixels
    
    handles.speedtext = uicontrol(handles.panel,'Style', 'text',...
        'Units','pixels','Position',[5,handles.uipanel_height/2*0.1,35,handles.uipanel_height/2*0.8],'HorizontalAlignment','right');
    handles.speededit = uicontrol(handles.panel,'Style', 'edit',...
        'Units','pixels','Position',[5+35,handles.uipanel_height/2*0.1+1,50,handles.uipanel_height/2*0.8]);
    handles.repeatbutton = uicontrol(handles.panel,'Style', 'checkbox',...
        'Units','pixels','Position',[5+35+50+20,handles.uipanel_height/2*0.1+2,65,handles.uipanel_height/2*0.8]);
    handles.timetext = uicontrol(handles.panel,'Style', 'text',...
        'Units','pixels','Position',[5+35+50+20+65,handles.uipanel_height/2*0.1,fig_pos(3)-(5+35+50+20+65)-5,handles.uipanel_height/2*0.8],'HorizontalAlignment','right');
    handles.slider = uicontrol(handles.panel,'Style', 'slider','Value',1,...
        'Units','pixels','Position',[1,handles.uipanel_height/2,fig_pos(3)-(handles.startbutton_width*2)+1,handles.uipanel_height/2]);
    handles.playbutton = uicontrol(handles.panel,'Style', 'pushbutton',...
        'Units','pixels','Position',[fig_pos(3)-handles.startbutton_width*2+2,handles.uipanel_height/2,handles.startbutton_width-2,handles.uipanel_height/2]);
    handles.savebutton = uicontrol(handles.panel,'Style', 'pushbutton',...
        'Units','pixels','Position',[fig_pos(3)-handles.startbutton_width+2,handles.uipanel_height/2,handles.startbutton_width-2,handles.uipanel_height/2]);
    set(fig,'ResizeFcn',@(hObject, eventdata, handles) animator(fig,'animator_ResizeFcn'));
else
    handles.speedtext    = handles.options.UI.speedtext;
    handles.speededit    = handles.options.UI.speededit;
    handles.repeatbutton = handles.options.UI.repeatbutton;
    handles.timetext     = handles.options.UI.timetext;
    handles.slider       = handles.options.UI.slider;
    handles.playbutton   = handles.options.UI.playbutton;
    handles.savebutton   = handles.options.UI.savebutton;
end

% set UI properties
set(handles.speedtext, 'String','Speed:');
set(handles.speededit, 'Callback',@(hObject, eventdata) animator(fig,'animator_speededitFcn'),...
    'String',handles.options.speed);
set(handles.repeatbutton,'String','Repeat');
set(handles.slider,'Value',1,'Callback',@(hObject, eventdata) animator(fig,'animator_sliderFcn'),...
    'Min',1,'Max',numel(tspan),'Enable','on');
set(handles.playbutton, 'String','play','Enable','on',...
    'Callback',@(hObject, eventdata) animator(fig,'animator_starttimerFcn'));
set(handles.savebutton, 'String','save','Enable','on',...
    'Callback',@(hObject, eventdata) animator(fig,'animator_saveFcn'));

% create timer
handles.guiTimer = timer;
set(handles.guiTimer,'Period',0.05,'ExecutionMOde','FixedRate','StartDelay',0.1,'TimerFcn',@(hObject, eventdata) animator(fig,'animator_timerFcn'));

% set close and resize function
set(fig,'CloseRequestFcn',@(hObject, eventdata, handles) animator(fig,'animator_CloseRequestFcn'));

% find figures with the same tspan data
if handles.options.timelines
    h_lines = findobj('XData',tspan); % get all objects that use tspan data as XData
    if ~isempty(h_lines)
        h_axes = get(h_lines,'Parent'); % get axes by lines
        if iscell(h_axes), h_axes = [h_axes{:}]; end % convert for cell array into vector
        h_axes = unique(h_axes); % remove double entries
    else
        h_axes = [];
    end
    handles.timelines = [];
    for n=1:length(h_axes)
        handles.timelines(n) = line('Parent',h_axes(n), 'Color', 'k', 'LineWidth',1,'Xdata',handles.tspan(1)*[1,1],'Ydata',get(h_axes(n),'ylim'));
    end
end

% update handles
guidata(fig,handles)

% plot first frame
animator_updateFcn(fig)

% start timer
if handles.options.startplay
    animator_starttimerFcn(fig);
end

%% animator_CloseRequestFcn %%
function animator_CloseRequestFcn(fig)
% get handles
handles = guidata(fig);

% stop and delete timer
if isfield(handles,'guiTimer') && isvalid(handles.guiTimer)
    stop(handles.guiTimer);
    delete(handles.guiTimer);
end

% delete time lines
if handles.options.timelines
    for i=1:length(handles.timelines)
        if ishandle(handles.timelines(i))
            delete(handles.timelines(i));
        end
    end
end

% delete figure
delete(fig);

%% animator_ResizeFcn %%
function animator_ResizeFcn(fig)
% get handles
handles = guidata(fig);
fig_pos = get(fig,'Position');
timetext_pos = get(handles.timetext,'Position');
set(handles.figpanel,'Position',[0,handles.uipanel_height,fig_pos(3),max(fig_pos(4)-handles.uipanel_height,1)]);
set(handles.panel,'Position',[0,0,fig_pos(3),handles.uipanel_height]);
set(handles.slider,'Position',[1,handles.uipanel_height/2,fig_pos(3)-handles.startbutton_width*2+1,handles.uipanel_height/2]);
set(handles.playbutton,'Position',[fig_pos(3)-handles.startbutton_width*2+2,handles.uipanel_height/2,handles.startbutton_width-2,handles.uipanel_height/2]);
set(handles.savebutton,'Position',[fig_pos(3)-handles.startbutton_width+2,handles.uipanel_height/2,handles.startbutton_width-2,handles.uipanel_height/2]);
set(handles.timetext,'Position',[timetext_pos(1),timetext_pos(2),max(fig_pos(3)-timetext_pos(1)-5,1),timetext_pos(4)]);
%% animator_updateFcn %%
function animator_updateFcn(fig)
% get handles
handles = guidata(fig);

% get frame number
n = floor(get(handles.slider,'Value'));

% run plot function
handles.plot_fun(n);

% update time/frame string
str = sprintf('Time:%.2f/%.2f Frame:%i/%i',handles.tspan(n),max(handles.tspan),n,numel(handles.tspan));
set(handles.timetext,'String',str)

% update time lines
if handles.options.timelines
    for i=1:length(handles.timelines)
        if ishandle(handles.timelines(i))
            set(handles.timelines(i),'Xdata',handles.tspan(n)*[1,1],'Ydata',get(get(handles.timelines(i),'Parent'),'ylim'));
        end
    end
end

%% animator_sliderFcn %%
function animator_sliderFcn(fig)
% get handles
handles = guidata(fig);

% reset counter
if strcmp(handles.guiTimer.Running,'on')
    n = floor(get(handles.slider,'Value'));
    handles.starttime = handles.tspan(n);
    handles.counter = tic;
    guidata(fig,handles)
end

% update figure
animator_updateFcn(fig)

%% animator_timerFcn %%
function animator_timerFcn(fig)
% get handles
handles = guidata(fig);

% get speed
speed = str2double(get(handles.speededit,'string'));

% get time
time = toc(handles.counter)*speed+handles.starttime;

% determine frame
frame = find(handles.tspan>time,1,'first');
if isempty(frame) % reached the end of the simulation
    if get(handles.repeatbutton,'Value')
        frame = 1; % first frame
        handles.starttime = handles.tspan(1); % reset counters
        handles.counter = tic;
        guidata(fig,handles);
    else
        frame = numel(handles.tspan); % last frame
        animator_starttimerFcn(fig); % stop simulation
    end
end
if time<handles.tspan(1) % reached the start of the simulation (due to negative speed)
    if get(handles.repeatbutton,'Value')
        frame = numel(handles.tspan); % last frame
        handles.starttime = handles.tspan(end); % reset counters
        handles.counter = tic;
        guidata(fig,handles);
    else
        frame = 1; % first frame
        animator_starttimerFcn(fig); % stop simulation
    end
end

% set slider
set(handles.slider,'Value',frame)

% update plot
animator_updateFcn(fig)

%% animator_starttimerFcn %%
function animator_starttimerFcn(fig)
% get handles
handles = guidata(fig);

if strcmp(handles.guiTimer.Running,'off')
    % get start time
    n = floor(get(handles.slider,'Value'));
    if n==length(handles.tspan);
        n = 1; % first frame
        set(handles.slider,'Value',n)
    end
    handles.starttime = handles.tspan(n);
    handles.counter = tic;% start counter
    guidata(fig,handles)
    
    % start timer
    start(handles.guiTimer);
    set(handles.playbutton,'String','stop');
    
    animator_timerFcn(fig)
else
    stop(handles.guiTimer);
    set(handles.playbutton,'String','play');
end

%% animator_speededitFcn %%
function animator_speededitFcn(fig)
% get handles
handles = guidata(fig);

% reset timer
n = floor(get(handles.slider,'Value'));
handles.starttime = handles.tspan(n);
handles.counter = tic;
guidata(fig,handles)

%% animator_saveFcn %%
function animator_saveFcn(fig)
% get handles
[movieName, moviePath] = uiputfile();
pause(.1)

if movieName
    handles = guidata(fig);
    
    % set frame rate
    frameRate = 24;
    
    % set the time frame for the movie
    tMovie = handles.tspan(1):1/frameRate:handles.tspan(end);
    for i = 1:length(tMovie)
        [~, n] = min(abs(handles.tspan-tMovie(i)));
        handles.starttime = handles.tspan(n);
        set(handles.slider,'Value',n)
        handles.counter = tic;
        guidata(fig,handles)
        
        % update figure and store the frame
        animator_updateFcn(fig)
        res = get(0, 'ScreenPixelsPerInch');
        fig2 = figure('Position', [0 0 1280 720], 'PaperUnits', 'inches','PaperSize',[1280/res,720/res],'PaperPosition',[0 0 1280/res 720/res],'visible','off');
        newpanel = copyobj(handles.figpanel, fig2);
        set(newpanel, 'Position', [0 0 1280 720], 'BorderWidth', 0, 'BackgroundColor', [1 1 1]);        
        print(fig2, [moviePath filesep movieName '_' num2str(i) '.png'], '-dpng', '-r96')
        close(fig2)
    end
    
    % % save the movie
    % movie2avi(movieFrames, [moviePath movieName], 'compression', 'none', 'fps', frameRate);
end

%% animator_options %%
function options = animator_options(varargin)

if mod(length(varargin),2)==1
    error('animator:options','Invalid options pairs');
end

% default options
options.speed = 1;
options.startplay = true;
options.timelines = true;
options.UI = [];

for i=1:2:length(varargin)
    switch varargin{i}
        case 'Speed'
            options.speed = varargin{i+1};
        case 'StartPlay'
            options.startplay = varargin{i+1};
        case 'TimeLines'
            options.timelines = varargin{i+1};
        case 'UI'
            options.UI = varargin{i+1};
        otherwise
            error('animator:options','Unknown option: %s',varargin{i});
    end
end