function energy(mdl)
%ENERGY outputs energy estimates for a walkiing simulation
%   INPUT: A multibody structure with simulation results (mdl).
%
% created by: Wietse van Dijk, Delft University of Technology (2013)
mdl.initmatrix;

% take the last nsteps of the simulation
steps = [];
for i = 2:length(mdl.simt)-(0.1/mdl.dt)
    if any(mdl.simrec(mdl.findcontrol('contactforce').recid([2 5]),i)) ...
            && any(mdl.simrec(mdl.findcontrol('contactforce').recid([2 5]),i+(0.1/mdl.dt))) ...
            && ~any(mdl.simrec(mdl.findcontrol('contactforce').recid([2 5]),i-1))
        steps = [steps i];
    end
end
range = steps(end-5):steps(end);

% find the mass of the walker
diagM = diag(mdl.simcons.M);
M = sum(diagM(1:3:end));

% joint positions and velocities
qid = [];
qidp = [];
for i = 1:length(mdl.findcontrol('reflex').motors)
    qid(end+1) = mdl.findcontrol('reflex').motors{i}.qid;
    qidp(end+1) = mdl.findcontrol('reflex').motors{i}.parent.qid(end);
end

relpos = mdl.simx(qid,:) - mdl.simx(qidp,:);
relvel = mdl.simx(qid+9,:) - mdl.simx(qidp+9,:);

%% muscle energy
[E, muscles] = muscleenergyexperditure(mdl);

muscleEnergy = mean(E.total(range))/M;
muscleWork = mean([[muscles.m] [muscles.m]] * abs(E.mech(:,range)))/M;

ankleMuscleEnergy = mean([[muscles(1:3).m] [muscles(1:3).m]] * ...
    (E.act([1 2 3 8 9 10], range) + E.sl([1 2 3 8 9 10], range) + E.mech([1 2 3 8 9 10], range)))/M;
ankleMusclework = mean([[muscles(1:3).m] [muscles(1:3).m]] * abs(E.mech([1 2 3 8 9 10],range)))/M;

%% joint work
% joint work/torque
torquecontrollers = {'reflex', 'ligaments'};
tor = zeros(2,6,length(range));
ntor = 0;
for iC = 1:length(torquecontrollers)
    if ~isempty(mdl.findcontrol(torquecontrollers{iC}))
        ntor = ntor + 1;
        tor(ntor, :, :) = mdl.simrec(mdl.findcontrol(torquecontrollers{iC}).recid(end-5:end), range)/M;
    end
end

jointWork = sum(sum(abs(relvel(:, range) .* squeeze(sum(tor,1))))) / numel(range);
ankleWork = sum(sum(abs(relvel([3 6], range) .* squeeze(sum(tor(:, [3 6], :),1))))) / numel(range);

disp('-----------------------------------------')
disp('energy measures over all joints: ')
disp(['estimated metabolic rate: ' num2str(muscleEnergy) ' W/kg'])
disp(['avarage muscle power: ' num2str(muscleWork) ' W/kg'])
disp(['avarage absolute joint power: ' num2str(jointWork) ' W/kg'])
disp('-----------------------------------------')
disp('energy measures ankle only:')
disp(['estimated metabolic rate: ' num2str(ankleMuscleEnergy) ' W/kg'])
disp(['avarage muscle power: ' num2str(ankleMusclework) ' W/kg'])
disp(['avarage absolute joint power: ' num2str(ankleWork) ' W/kg'])
end

