% this is an example file that allows you to build multiple models. This
% file should be run using cell mode. To run one cell at a time use 
% Crtl + Enter. First evaluate the frist cell to initiate the model and 
% clear the workspace. Then run only one of the middle cells to build a 
% model. Run the last cell to simulate and animate the model.

%% init multibody
% evaluate this cell (Ctrl + Enter) to initialize the model
initsim % add the necessary directories to the search path

%% falling mass
close all; clear all; mdl = multibody; % create a multibody object
% evaluatate this one of these three cells (falling mass, double pendulum,
% or 4-link mechanism with constraint

% this cell makes a falling mass
mdl.makepart(floatjoint('parent', mdl.parts{1}, 'pos', [0; 1; 0], 'name', 'aaa'));
mdl.makepart(mass('parent', mdl.parts{2}, 'pos', [0; 1; 0], 'm', [1; 1; 0.1]));
mdl.makepart(point('parent', mdl.parts{2}, 'pos', [0; 1; 0]));
mdl.preprocess;

c = contact;
c.points = mdl.points(1);
mdl.makecontroller(c);

mdl.makecontroller(gravity);

%% double pendulum
close all; clear all; mdl = multibody; % create a multibody object
% this cell makes a double pendulum
mdl.makepart(rotjoint('parent', mdl.parts{1}, 'pos', [0; 1.5; 0], 'name', 'j1', 'q0', 0.1));
mdl.makepart(mass('parent', mdl.parts{2}, 'pos', [0; 1; 0], 'm', [1; 1; 0.1], 'name', 'm1'));
mdl.makepart(point('parent', mdl.parts{2}, 'pos', [0; 1; 0]));
mdl.makepart(rotjoint('parent', mdl.parts{2}, 'pos', [0; 1; 0], 'name', 'j2'));
mdl.makepart(mass('parent', mdl.parts{5}, 'pos', [0; 0.5; 0], 'm', [1; 1; 0.1], 'name', 'm2'));
mdl.preprocess;

c = contact;
c.points = mdl.points(1);
mdl.makecontroller(c);

mdl.makecontroller(gravity);

%% 4-link mechanism with contraint
close all; clear all; mdl = multibody; % create a multibody object
% this cell makes a 4 link mechanism with a constraint
mdl.makepart(rotjoint('parent', mdl.parts{1}, 'pos', [0; 2; 0], 'name', 'j1'));
mdl.makepart(mass('parent', mdl.parts{2}, 'pos', [-.25; 1.75; 0], 'm', [1; 1; 0.1], 'name', 'm1'));
mdl.makepart(rotjoint('parent', mdl.parts{2}, 'pos', [-.5; 1.5; 0], 'name', 'j2'));
mdl.makepart(mass('parent', mdl.parts{4}, 'pos', [-.25; 1.25; 0], 'm', [1; 1; 0.1], 'name', 'm2'));

mdl.makepart(rotjoint('parent', mdl.parts{1}, 'pos', [0; 2; 0], 'name', 'j3'));
mdl.makepart(mass('parent', mdl.parts{6}, 'pos', [.25; 1.75; 0], 'm', [1; 1; 0.1], 'name', 'm3'));
mdl.makepart(rotjoint('parent', mdl.parts{6}, 'pos', [.5; 1.5; 0], 'name', 'j4'));
mdl.makepart(mass('parent', mdl.parts{8}, 'pos', [.25; 1.25; 0], 'm', [1; 1; 0.1], 'name', 'm4'));
mdl.makepart(point('parent', mdl.parts{8}, 'pos', [0; 1; 0]));

mdl.makepart(constraint('parent', mdl.parts{4}, 'child', mdl.parts{8}, 'pos', [0; 1; 0]));
mdl.q_dep = [0 1 0 1];
mdl.preprocess;

c = contact;
c.points = mdl.points(1);
mdl.makecontroller(c);

mdl.makecontroller(gravity);

%% run simulation
% run the simulation
mdl.t = [0 5]; % run it five seconds
mdl.preprocess; % preprocess the model
mdl.simulate(0, 0); % simulate
mdl.setshape; % initialized the shape of the parts
mdl.makefigure; % make a figure for the animation
mdl.animate; % animate
