function mdl = runpso_reflex(varargin)
%WALKUPDATE Summary of this function goes here
%   Detailed explanation goes here

if nargin == 0
    makemex = 1;
else
    makemex = varargin{1};
end

% initialize the random number generator
rng(int32(sum(clock) * 100))

% build the model
mdl = buildmodel;

% add noise to the muscle model
minfreq = 5;
maxfreq = 10;
maxt = 20;
mdl.findcontrol('reflex').cons.noiseX = [zeros(14,1) cumsum(rand(14, maxt*maxfreq)*(1/minfreq-1/maxfreq)+1/maxfreq, 2)];
mdl.findcontrol('reflex').cons.noiseY = rand(size(mdl.findcontrol('reflex').cons.noiseX));
mdl.findcontrol('reflex').cons.maxnoise = 0.02;

% complile code if applicable
if makemex
    mdl.makemex % make a mex file
    disp('code compiled')
end

% save the workspace
save(['generated' filesep 'workspace'], 'mdl') % save the generated model
disp('model saved')

% generate file name
c = clock;
savename = fullfile('results', ['human_reflex' date '_' num2str(c(4)) '-' num2str(c(5)) '-' num2str(round(c(6)))]);

% set the parameters to be optimized
opt = [];
opt(end + 1).type = 'leanangle'; opt(end).bound = [-15/180*pi 0]; opt(end).initbound = [-15/180*pi 0];
opt(end + 1).type = 'legangle'; opt(end).bound = [25/180*pi 40/180*pi]; opt(end).initbound = [25/180*pi 40/180*pi];
opt(end + 1).type = 'swingspeed'; opt(end).bound = [-.9 .1]; opt(end).initbound = [-.9 .1];

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.phi_knee_off'; opt(end).bound = [-60 0] / 180 * pi; opt(end).initbound = [-12 -0] / 180 * pi; % 4
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.k_phi'; opt(end).bound = [0 10]; opt(end).initbound = [0 3]; % 5
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.dS'; opt(end).bound = [.05 1]; opt(end).initbound = [.1 .3]; % 6
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.theta_ref'; opt(end).bound = [0.017 0.11]; opt(end).initbound = [0.08 0.105]; % 7
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.k_p'; opt(end).bound = [0.5 10]; opt(end).initbound = [1 3]; % was 1.78 22 % 8
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.k_d'; opt(end).bound = [.1 0.75]; opt(end).initbound = [.1 .25]; % was .2 .3 % 9
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.k_bw'; opt(end).bound = [.05 5]; opt(end).initbound = [.5 1.5]; % 10
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.k_lean'; opt(end).bound = [1 5.7]; opt(end).initbound = [1 1.6]; % 11
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.S_0_bal'; opt(end).bound = [0.01 0.32]; opt(end).initbound = [0.02 0.08]; % 12

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(1).G'; opt(end).bound = [0.8 2.17] / mdl.findcontrol('reflex').cons.m(1).F_max; opt(end).initbound = [.8 1.1] / mdl.findcontrol('reflex').cons.m(1).F_max; % 13

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(2).G'; opt(end).bound = [0.55 3.2] / mdl.findcontrol('reflex').cons.m(2).l_opt; opt(end).initbound = [1 1.5] / mdl.findcontrol('reflex').cons.m(2).l_opt; % muscle reflex gain 14
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(2).l_off'; opt(end).bound = [0.59 0.8] * mdl.findcontrol('reflex').cons.m(2).l_opt; opt(end).initbound = [0.63 0.7] * mdl.findcontrol('reflex').cons.m(2).l_opt;% length offset 15
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(2).G2'; opt(end).bound = [0 5] / mdl.findcontrol('reflex').cons.m(1).F_max; opt(end).initbound = [.6 1.6] / mdl.findcontrol('reflex').cons.m(1).F_max;% muscle reflex gain soleus - tibialis anterior coupling 16

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(3).G'; opt(end).bound = [0 10] / mdl.findcontrol('reflex').cons.m(3).F_max; opt(end).initbound = [.8 3] / mdl.findcontrol('reflex').cons.m(3).F_max; % 17

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(4).G'; opt(end).bound = [0 10] / mdl.findcontrol('reflex').cons.m(4).F_max; opt(end).initbound = [.5 2] / mdl.findcontrol('reflex').cons.m(4).F_max; % muscle reflex gain 18
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(4).S_0'; opt(end).bound = [0.047 0.71]; opt(end).initbound = [0.05 0.15]; % initial stimulation % in model 0.08 in artikel 0.09 19

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(5).G'; opt(end).bound = [0 0.67] / mdl.findcontrol('reflex').cons.m(5).F_max; opt(end).initbound = [0.5 0.67] / mdl.findcontrol('reflex').cons.m(5).F_max; % 20
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(5).l_off'; opt(end).bound = [0.6 5] * mdl.findcontrol('reflex').cons.m(5).l_opt; opt(end).initbound = [0.6 0.9] * mdl.findcontrol('reflex').cons.m(5).l_opt; % 21

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(6).G'; opt(end).bound = [0 0.55] / mdl.findcontrol('reflex').cons.m(6).F_max; opt(end).initbound = [0.25 0.45] / mdl.findcontrol('reflex').cons.m(6).F_max; % 22

opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(7).G'; opt(end).bound = [.17 3] / mdl.findcontrol('reflex').cons.m(7).l_opt; opt(end).initbound = [.5 1] / mdl.findcontrol('reflex').cons.m(7).l_opt; % 23
opt(end + 1).type = 'mdl.findcontrol(''reflex'').cons.m(7).G2'; opt(end).bound = [0 20] / mdl.findcontrol('reflex').cons.m(5).l_opt; opt(end).initbound = [2 8] / mdl.findcontrol('reflex').cons.m(5).l_opt; % 24

% set the bounds to a vector
xbound = zeros(length(opt),2);
xinitbound = zeros(length(opt),2);
for i = 1:length(opt)
    xbound(i,:) = opt(i).bound;
    xinitbound(i,:) = opt(i).initbound;
end

% initialize and run the PSO
p = pso('posmin', xbound(:,1)', ...
    'posmax', xbound(:,2)', ...
    'initposmin', xinitbound(:,1)', ...
    'initposmax', xinitbound(:,2)', ...
    'costfunc', @(x) objectivemultirun(x, opt, 3, 'min'), ...
    'iterations', 60, ...
    'popsize', 50, ...
    'initsize', 100, ...
    'localsize', 3, ...
    'socialsize', 2, ...
    'output', {'file', 'text'}); % , 'graphic'});
p.savefile = savename;
p.run;

% find the best particle and save the siulation file
[~, ~, ~, mdl] = objectivemultirun(p.overalbestpos(p.iterations), opt, 10, 'max');
mdl.resample(0.005); % resample the model to save space
mdl.findcontrol('reflex').rec0 = [];
save([p.savefile '_mdl'], 'mdl')
end

function [errcode, fitnessout, stageout, mdl] = objectivemultirun(x, opt, nRuns, minmax)

fitnessoutMulti = zeros(nRuns,1);
stageoutMulti = zeros(nRuns,1);

for i = 1:nRuns
    [errcode, fitnessoutMulti(i), stageoutMulti(i), mdl] = objectivefunc(x, opt);
end

if strcmp(minmax, 'min')
    stageout = min(stageoutMulti);
    fitnessout = min(fitnessoutMulti(stageoutMulti == stageout));
else
    stageout = max(stageoutMulti);
    fitnessout = max(fitnessoutMulti(stageoutMulti == stageout));
end

end

function [errcode, fitnessout, stageout, mdl] = objectivefunc(x, opt)
% OBJECTIVE FUNCTION runs a simulation and calculates a fitness
m = load('workspace');
mdl = m.mdl;

errcode = 0; % errorcode not used in this type of simulation

% set the parameters according to the particle position
for iopt = 1:length(opt)
    switch opt(iopt).type
        case 'leanangle'
            mdl.findpart('origin').q0 = [0; 0; x(iopt)];
            mdl.findpart('leftknee').q0 = x(iopt);
        case 'legangle'
            legangle1 = x(iopt);
            legangle2 = x(iopt) + (5/180*pi);
            mdl.findpart('righthip').q0 = legangle1;
            mdl.findpart('rightknee').q0 = legangle2;
            mdl.findpart('rightankle').q0 = legangle2;
        case 'swingspeed'
            mdl.findpart('righthip').dq0 = x(iopt);
            mdl.findpart('rightknee').dq0 = x(iopt);
            mdl.findpart('rightankle').dq0 = x(iopt);
        otherwise
            eval([opt(iopt).type ' = ' num2str(x(iopt)) ';']);
    end
end

% add noise to the muscle model
minfreq = 5;
maxfreq = 10;
maxt = 20;

mdl.findcontrol('reflex').cons.noiseX = [zeros(14,1) cumsum(rand(14, maxt*maxfreq)*(1/minfreq-1/maxfreq)+1/maxfreq, 2)];
mdl.findcontrol('reflex').cons.noiseY = rand(size(mdl.findcontrol('reflex').cons.noiseX));

mdl.initmatrix; % update the model constants

% place the model on the floor
[~, ~, Tp] = Tmatrices(mdl.x0, mdl.dx0, mdl.simcons.p_in);
mdl.x0(2) = -min(Tp(2:3:end));

% run the simulation
mdl.simulate(1, 0);

% calculate the different fitness criteria
nSteps = 7;
steps = [];
for i = 2:length(mdl.simt)-(0.1/mdl.dt)
    if any(mdl.simrec(mdl.findcontrol('contactforce').recid([2 5]),i)) ...
            && any(mdl.simrec(mdl.findcontrol('contactforce').recid([2 5]),i+(0.1/mdl.dt))) ...
            && ~any(mdl.simrec(mdl.findcontrol('contactforce').recid([2 5]),i-1))
        steps = [steps i];
    end
end
if length(steps) > (nSteps + 1)
    range = steps(end-nSteps):steps(end);
else
    range = 1:length(mdl.simt);
end

f = walkcostfunction(mdl, range);

fitness = []; stage = [];

% model doesn't fall
fitness(end + 1) = f.time;
stage(end + 1) = fitness(end) > mdl.t(2) - 0.01;
% model has roughly the correct speed
fitness(end + 1) = -abs(f.speed - 1.2);
stage(end + 1) = fitness(end) > -0.5;
% model has a uniform steplength
fitness(end + 1) = -f.steplengthstd;
stage(end + 1) = fitness(end) > -0.05;
% finetuning of the speed
fitness(end + 1) = -abs(f.speed - 1.2);
stage(end + 1) = fitness(end) > -0.1;
% optimize for torque
fitness(end + 1) = -(f.musclework * f.activation * f.ligamentstor);

% find the highest stage
stageout = find(~stage, 1) - 1;
if isempty(stageout)
    stageout = length(stage);
end

% set the fitness of the highest stage
fitnessout = fitness(stageout + 1);
end

function mdl = buildmodel
mdl = buildhuman_newfoot; % make a human multibody system
mdl.t = [0 15]; % max simulation time
mdl.preprocess; % preprocess the structure for optimization
end

