function makews(varargin)
% MAKEWS creates a workspace that is used by the optimizatione framework
%   MAKEWS(makemex, modeltype)
%   IN:
%   makemex: 0-1 deciding if mex files should be created
%   modeltype: string deciding what type of model is build

initsim

if nargin >= 1
    makemex = varargin{1};
else
    makemex = 1;
end

if nargin >= 2
    modeltype = varargin{2};
else
    modeltype = 'human';
end

switch modeltype
    case 'human'
        mdl = buildhuman_newfoot;
        mdl.t = [0 15];
        range = mdl.t(1):0.001:mdl.t(2);
        mdl.findcontrol('reflex').cons.noise = zeros(14, length(range));
        mdl.preprocess;
        if makemex
            mdl.makemex;
        end
    case 'futureattract'
        load human_init;
        % add future attract controler
        c = futureattract;
        c.motors = mdl.parts([4 6 8 16 18 20]);
        c.cons.shift = 0.1 * 2 * pi *  ones(6, 1); % phase shift
        c.cons.k = 1 * ones(6, 1); % stiffness
        c.cons.d = 0.01 * ones(6, 1); % damping
        
        c.cons.shiftswing = 0.1 * 2 * pi *  ones(6, 1); % phase shift
        c.cons.kswing = 0 * ones(6, 1); % stiffness
        c.cons.dswing = 0 * ones(6, 1); % damping
        
        c.contacts = mdl.controllers{1};
        
        c.update;
        c.s0(2) = 4.88;
        mdl.makecontroller(c);
        
        range = 0:0.001:20;
        mdl.findcontrol('reflex').cons.noise = zeros(14, length(range));
        
        mdl.preprocess;
        mdl.t = [0 10];
        mdl.makemex;
        mdl.simulate(1, 0);
        
        mdl.findcontrol('reflex').cons.maxnoise = 0.02;
        mdl.t = [0 20];
        mdl.findcontrol('futureattract').cons.on = 0;
        mdl.initmatrix;
    case 'feedforward'
        load human_init;
        
        % feed forward control
        c = feedforwardkernels;
        c.motors = mdl.parts([4 6 8 16 18 20]);
        c.cons.on = 1;
        c.cons.filter.w = zeros(20, 3);
        c.update;
        mdl.makecontroller(c);
        
        % noise
        range = 0:0.001:20;
        mdl.findcontrol('reflex').cons.noise = zeros(14, length(range));
        
        mdl.preprocess;
        mdl.initmatrix
        
        mdl.makemex;
    case 'nosupport'
        load walking_new
        
        mdl.preprocess;
        mdl.initmatrix;
        
        mdl.makemex;
        
    case {'ffgaitpattern', 'ffgaitpatternankle'}
        % load walking_new_umberger
        load walking_new
        
        % phase detection
        c = phasedetect;
        c.contacts = mdl.findcontrol('contactforce');
        c.motors = mdl.parts([4 16]);
        mdl.makecontroller(c);
        
        % feed forward form gait pattern
        c = ffgaitpattern;
        c.gaitphase = mdl.findcontrol('phase_detect');
        
        switch modeltype
            case 'ffgaitpattern'
                % actuate all joints
                c.motors = mdl.parts([4 6 8 16 18 20]);
            case 'ffgaitpatternankle'
                % actuate only the ankle
                c.motors = mdl.parts([4 6 8 16 18 20]);
                c.cons.winterjoints = 3;
                % -------------
                % load ankle torque to match the data laurent used for his
                % experiment
                load ankletorquesteve;
                c.cons.w.t(:,3) = ankleTorqueSteve;
                % -------------
        end
        
        mdl.makecontroller(c);
                
        mdl.preprocess;
        mdl.initmatrix;
        
        mdl.makemex;
        
    case 'topo10_human'
        mdl = buildtopo10_no_toes;
        mdl.t = [0 15];
        range = 0:0.001:20;
        mdl.findcontrol('reflex').cons.noise = zeros(14, length(range));
        mdl.preprocess;
        mdl = solveinitcondition(mdl);
        if makemex
            mdl.makemex;
        end
    case 'topo10_human_wr'
        load muscle_top10
        % add future attract controler
        c = futureattract;
        c.motors = mdl.parts([10 11 16 17]);
        
        c.cons.shift = zeros(4, 1) + 0.1; % phase shift
        c.cons.k = zeros(4, 1) + 1; % stiffness
        c.cons.d = zeros(4, 1); % damping
        
        c.cons.shiftswing = 0 * 2 * pi *  ones(4, 1); % phase shift
        c.cons.kswing = 0 * ones(4, 1); % stiffness
        c.cons.dswing = 0 * ones(4, 1); % damping
        
        c.cons.leftlearn = 1; % index of the left motor to learn the phase
        c.cons.rightlearn = 3; % index of the right motor ro learn the phase
        
        c.contacts = mdl.controllers{1};
        
        c.update;
        c.s0(2) = 4.88;
        mdl.makecontroller(c);
        
        range = 0:0.001:20;
        mdl.findcontrol('reflex').cons.noise = zeros(14, length(range));
        
        mdl.preprocess;
        mdl = solveinitcondition(mdl);
        mdl.t = [0 10];
        mdl.makemex;
        mdl.simulate(1, 0);
        
        mdl.findcontrol('reflex').cons.maxnoise = 0.02;
        mdl.t = [0 20];
        mdl.findcontrol('futureattract').cons.on = 1;
        mdl.findcontrol('futureattract').cons.swingstance = 0;
        mdl.initmatrix;   
    case 'ferris_support'
        load walking_new
        c = ferrissupport;
        c.motors = mdl.parts([4 6 8 16 18 20]);
        c.reflex = mdl.findcontrol('reflex');
        c.cons.gain = 10;
        mdl.makecontroller(c);
        
        mdl.preprocess;
        mdl.initmatrix;
        
        if makemex
            mdl.makemex;
        end
end

ws.mdl = mdl;
ws.expand = 1;

save(['generated' filesep 'workspace.mat'], '-struct', 'ws');
end

% vi:ts=4:et
