function [E, muscles] = muscleenergyexperditure(mdl)
%MUSCLEENERGYEXPERDITURE Summary of this function goes here
%   Detailed explanation goes here

% fixed model parameters
N = 1.5; % eccentric force enehancement []
K = 5; % cusvature constant []
eps_ref = 0.04; % refernece strain of the series elastic element []
c = 0.05; % residual force factor
rho = 1059.7; % musle density [kg/m^3];
sigma = 0.25 * 10^6; % muscle strain

t_ecc = 0.01; % exitation contraction coupling constant [s]
reflex = mdl.findcontrol('reflex'); % handle to the reflex controller
muscles = reflex.cons.m;

act = mdl.simx(mdl.nq * 2 + reflex.sid(1:14),:); % muscle activation
% stimulation not used since the activation constant is very low
stim = diff(act, [], 2) / mdl.dt * t_ecc + act(:, 1:end-1); % muscle stimulation

A_AM = act.^0.6;
A_S = act.^2;

alpha_s_st = 4 * 25 / (12 / 2.5);
alpha_s_ft = 1 * 153 / 12;
alpha_l = 4 * alpha_s_st;

f = mdl.simrec(mdl.findcontrol('reflex').recid(1:14), :);
l = mdl.simx(mdl.findcontrol('reflex').sid(15:28) + mdl.nq * 2, :);
s = diff(l,[],2)./mdl.dt;

% extended muscle properties
% soleus
muscles(1).FT = 0.2; % fraction of fast twitch muscle
muscles(1).A_rel = 0.18; % fraction of fast twitch muscle
muscles(1).B_rel = 2.16; % fraction of fast twitch muscle
% tibialis anterior
muscles(2).FT = 0.25; % fraction of fast twitch muscle
% gastrocnemius
muscles(3).FT = 0.5; % fraction of fast twitch muscle
% vastus
muscles(4).FT = 0.5; % fraction of fast twitch muscle
% hamsting
muscles(5).FT = 0.35; % fraction of fast twitch muscle
% gluteus
muscles(6).FT = 0.45; % fraction of fast twitch muscle
% hip flexors
muscles(7).FT = 0.5; % fraction of fast twitch muscle

S = 1; % scaling factor anaerobic

E.act = zeros(size(l)); % activation energy
E.sl = zeros(size(s)); % shortening lengthening energy
E.mech = zeros(size(s)); % mechanical work

for iMuscle = 1:14
    m = muscles(mod(iMuscle-1,7)+1); % the current muscle
    m.mass = m.F_max/sigma * rho * m.l_opt;
    muscles(mod(iMuscle-1,7)+1).m = m.mass;
    w = 0.56*m.l_opt; % wimdl.dth [m]
    
    dh_AM = 128 * m.FT + 25;
    F_iso = exp(log(c).*abs((l(iMuscle,:)-m.l_opt)./w).^3);
        
    E.act(iMuscle,:) = dh_AM * A_AM(iMuscle, :) * S .* (l(iMuscle,:) <= m.l_opt) + ...
        dh_AM * (0.4 + 0.6 * F_iso) .* act(iMuscle) * S .* (l(iMuscle,:) > m.l_opt);
    
    E.sl(iMuscle,:) = (-alpha_s_st * (1 - m.FT) - alpha_s_ft * m.FT) * (s(iMuscle,:)/m.l_opt) .* A_S(iMuscle, 2:end) * S .* (s(iMuscle, :) <= 0) + ...
        alpha_l * (s(iMuscle, :)/m.l_opt) .* act(iMuscle, 2:end) * S .* (s(iMuscle, :) > 0);
    
    E.sl(iMuscle,:) = E.sl(iMuscle,:) .* (l(iMuscle,2:end) <= m.l_opt) + ...
        E.sl(iMuscle,:) .* F_iso(2:end) .* (l(iMuscle,2:end) > m.l_opt);
        
    E.mech(iMuscle, :) = -f(iMuscle,2:end) .* s(iMuscle, :) / m.mass;        
end

E.total = [[muscles.m] [muscles.m]] * (E.act(:,2:end) + E.sl + E.mech);

end

